@@ -2,7 +2,7 @@ package URI;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = "1.51";
+$VERSION = "1.60";
 
 use vars qw($ABS_REMOTE_LEADING_DOTS $ABS_ALLOW_RELATIVE_SCHEME $DEFAULT_QUERY_FORM_DELIMITER);
 
@@ -90,6 +90,7 @@ sub _uric_escape
 {
     my($class, $str) = @_;
     $str =~ s*([^$uric\#])* URI::Escape::escape_char($1) *ego;
+    utf8::downgrade($str);
     return $str;
 }
 
@@ -218,6 +219,7 @@ sub opaque
     my $new_opaque = shift;
     $new_opaque = "" unless defined $new_opaque;
     $new_opaque =~ s/([^$uric])/ URI::Escape::escape_char($1)/ego;
+    utf8::downgrade($new_opaque);
 
     $$self = defined($old_scheme) ? $old_scheme : "";
     $$self .= $new_opaque;
@@ -243,6 +245,7 @@ sub fragment
     my $new_frag = shift;
     if (defined $new_frag) {
 	$new_frag =~ s/([^$uric])/ URI::Escape::escape_char($1) /ego;
+	utf8::downgrade($new_frag);
 	$$self .= "#$new_frag";
     }
     $old;
@@ -260,22 +263,13 @@ sub as_iri
 {
     my $self = shift;
     my $str = $$self;
-    if ($str =~ /\bxn--/ && $self->can("ihost")) {
-	my $ihost = $self->ihost;
-	if ($ihost) {
-	    my $u = $self->clone;
-	    $u->host("%%host%%");
-	    $str = $u->as_string;
-	    $str =~ s/%%host%%/$ihost/;
-	}
-    }
-    if ($str =~ s/%([89A-F][0-9A-F])/chr(hex($1))/eg) {
+    if ($str =~ s/%([89a-fA-F][0-9a-fA-F])/chr(hex($1))/eg) {
 	# All this crap because the more obvious:
 	#
 	#   Encode::decode("UTF-8", $str, sub { sprintf "%%%02X", shift })
 	#
-	# doesn't work.  Apparently passing a sub as CHECK only works
-	# for 'ascii' and similar direct encodings.
+	# doesn't work before Encode 2.39.  Wait for a standard release
+	# to bundle that version.
 
 	require Encode;
 	my $enc = Encode::find_encoding("UTF-8");
@@ -330,6 +324,8 @@ sub eq {
 sub abs { $_[0]; }
 sub rel { $_[0]; }
 
+sub secure { 0 }
+
 # help out Storable
 sub STORABLE_freeze {
        my($self, $cloning) = @_;
@@ -477,7 +473,7 @@ argument, it updates the corresponding component in addition to
 returning the old value of the component.  Passing an undefined
 argument removes the component (if possible).  The description of
 each accessor method indicates whether the component is passed as
-an escaped or an unescaped string.  A component that can be further
+an escaped (percent-encoded) or an unescaped string.  A component that can be further
 divided into sub-parts are usually passed escaped, as unescaping might
 change its semantics.
 
@@ -578,6 +574,11 @@ Returns a relative URI reference if it is possible to
 make one that denotes the same resource relative to $base_uri.
 If not, then $uri is simply returned.
 
+=item $uri->secure
+
+Returns a TRUE value if the URI is considered to point to a resource on
+a secure channel, such as an SSL or TLS encrypted one.
+
 =back
 
 =head1 GENERIC METHODS
@@ -876,12 +877,12 @@ methods and the generic query methods.  In addition, they support the
 following mailto-specific methods: $uri->to, $uri->headers.
 
 Note that the "foo@example.com" part of a mailto is I<not> the
-C<userinfo> and C<host> but instead the C<path>.  This allowed a
-mailto to contain multiple comma-seperated email addresses.
+C<userinfo> and C<host> but instead the C<path>.  This allows a
+mailto URI to contain multiple comma separated email addresses.
 
 =item B<mms>:
 
-The I<mms> URL specification can be found at L<http://sdp.ppona.com/>
+The I<mms> URL specification can be found at L<http://sdp.ppona.com/>.
 C<URI> objects belonging to the mms scheme support the common,
 generic, and server methods, with the exception of userinfo and
 query-related sub-components.
@@ -929,7 +930,7 @@ instead of TCP.  The syntax is the same as rtsp.
 
 =item B<rsync>:
 
-Information about rsync is available from http://rsync.samba.org.
+Information about rsync is available from L<http://rsync.samba.org/>.
 C<URI> objects belonging to the rsync scheme support the common,
 generic and server methods.  In addition, they provide methods to
 access the userinfo sub-components: $uri->user and $uri->password.
@@ -966,7 +967,7 @@ common, generic and server methods.
 
 =item B<ssh>:
 
-Information about ssh is available at http://www.openssh.com/.
+Information about ssh is available at L<http://www.openssh.com/>.
 C<URI> objects belonging to the ssh scheme support the common,
 generic and server methods. In addition, they provide methods to
 access the userinfo sub-components: $uri->user and $uri->password.
@@ -981,7 +982,7 @@ and the Namespace-Specific String respectively.
 The Namespace Identifier basically works like the Scheme identifier of
 URIs, and further divides the URN namespace.  Namespace Identifier
 assignments are maintained at
-<http://www.iana.org/assignments/urn-namespaces>.
+L<http://www.iana.org/assignments/urn-namespaces>.
 
 Letter case is not significant for the Namespace Identifier.  It is
 always returned in lower case by the $uri->nid method.  The $uri->_nid
@@ -1052,7 +1053,13 @@ delimited by ";" instead of "&" which is the default.
 
 =head1 BUGS
 
-Using regexp variables like $1 directly as arguments to the URI methods
+There are some things that are not quite right:
+
+=over
+
+=item *
+
+Using regexp variables like $1 directly as arguments to the URI accessor methods
 does not work too well with current perl implementations.  I would argue
 that this is actually a bug in perl.  The workaround is to quote
 them. Example:
@@ -1060,6 +1067,19 @@ them. Example:
    /(...)/ || die;
    $u->query("$1");
 
+
+=item *
+
+The escaping (percent encoding) of chars in the 128 .. 255 range passed to the
+URI constructor or when setting URI parts using the accessor methods depend on
+the state of the internal UTF8 flag (see utf8::is_utf8) of the string passed.
+If the UTF8 flag is set the UTF-8 encoded version of the character is percent
+encoded.  If the UTF8 flag isn't set the Latin-1 version (byte) of the
+character is percent encoded.  This basically exposes the internal encoding of
+Perl strings.
+
+=back
+
 =head1 PARSING URIs WITH REGEXP
 
 As an alternative to this module, the following (official) regular
@@ -1079,11 +1099,11 @@ L<URI::Split>, L<URI::Heuristic>
 RFC 2396: "Uniform Resource Identifiers (URI): Generic Syntax",
 Berners-Lee, Fielding, Masinter, August 1998.
 
-http://www.iana.org/assignments/uri-schemes
+L<http://www.iana.org/assignments/uri-schemes>
 
-http://www.iana.org/assignments/urn-namespaces
+L<http://www.iana.org/assignments/urn-namespaces>
 
-http://www.w3.org/Addressing/
+L<http://www.w3.org/Addressing/>
 
 =head1 COPYRIGHT
 
